#include <iostream>
#include <stdexcept>

#include <mysql.h>

class sql_error: public runtime_error {
public:
  sql_error(string name): runtime_error(name) {};
};

class mysql;
class mysql_table;

class mysql_row {
private:
  MYSQL *sql;
  MYSQL_RES *res;
  MYSQL_ROW row;
  unsigned long *lengths;
  void error() {throw sql_error(string(mysql_error(sql)));};
  
  mysql_row(MYSQL *the_sql,MYSQL_RES *the_res): sql(the_sql), res(the_res) {
    row=mysql_fetch_row(res);
    if (row==0) error();
    lengths=mysql_fetch_lengths(res);
  };

  friend class mysql_table;
  
public:
  int size() const {
    return mysql_num_fields(res);
  }
  string operator[](int n) const {
    return string(row[n],lengths[n]); 
  }
  
  string operator[](string field) const {
    int i;
    for (i=0;name(i)!=field && i<size();++i);
    if (i<size()) return (*this)[i];
    else return "";
  }

  string name(int n) const {
    mysql_field_seek(res,n);
    MYSQL_FIELD *field=mysql_fetch_field(res);
    return string(field->name);
  }
};

class mysql_table {
private:
  MYSQL *sql;
  MYSQL_RES *res;

  int seek_pos;
  void error() {throw sql_error(string(mysql_error(sql)));};
  
  mysql_table(MYSQL *the_sql, const char *query):
    sql(the_sql) {
    if (mysql_query(sql,query)!=0) 
      error();
    res=mysql_store_result(sql);
    if (res==0 && mysql_errno(sql)!=0) error();
    seek_pos=0;
  };

  friend class mysql;

public:
  int size() const {
    if (res==0) return 0;
    else return mysql_num_rows(res);
  }
  
  mysql_row operator[](int n) {
    if (seek_pos!=n) {
      mysql_data_seek(res,n);
      seek_pos=n;
    }
    seek_pos++;
    return mysql_row(sql,res);
  }
  ~mysql_table() {
    //free(res);
  };
};

class mysql {
private:
  MYSQL *sql;
public:
  void error() {throw sql_error(string(mysql_error(sql)));};
  
  mysql(const char *db=0, const char *user=0, 
	const char *host=0, const char *passwd=0,
	unsigned int port=0) {
    sql=mysql_init(0);
    if (!sql) throw sql_error(string("mysql_init failed"));
    if (!mysql_real_connect(sql,host,user,passwd,db,port,
			    0,0))
      error();
  };
  
  ~mysql() {
    mysql_close(sql);
  };
  
  mysql_table query(const char *s) {
    return(mysql_table(sql,s));
  };

  mysql_table query(const string &s) {
    return(mysql_table(sql,s.c_str()));
  };
};

